use std::collections::HashMap;

use crate::{common::Loc, tokeniser::tokentype::*};

use super::{typ::{Type, TypeBox}, Ast};

#[derive(Debug, Clone)]
pub struct ExprBox {
    inner: Expr,
    loc: Loc
}

impl ExprBox {
    pub fn new(loc: &Loc, expr: Expr) -> Self {
        Self { loc: loc.clone(), inner: expr }
    }
    pub fn inner(&self) -> &Expr {
        &self.inner
    }
    pub fn inner_mut(&mut self) -> &mut Expr {
        &mut self.inner
    }
    pub fn loc(&self) -> &Loc {
        &self.loc
    }
}

#[derive(Debug, Clone)]
pub enum Expr {
    // Comment(Comment),
    Group(Box<ExprBox>),
    UnOp {
        typ: Punctuation,
        right: Box<ExprBox>,
    }, 
    BinOp {
        typ: Punctuation,
        left: Box<ExprBox>,
        right: Box<ExprBox>,
    },
    Literal(super::literal::Literal),
    ArrayIndex {
        name: Box<ExprBox>,
        index: Box<ExprBox>,
    },
    Path(Path),
    Call {
        path: Box<ExprBox>,
        params: CallParams, // ExprBox ~ (, Expr)*
    },
    //MethodCall {
    //    var_name: Box<ExprBox>,
    //    method_name: Ident,
    //    params: CallParams,
    //},

    /// the left side only exists on the /.|->/ chain
    FieldAccess {
        left: Box<Option<ExprBox>>,
        right: Box<ExprBox>,
    },
    PtrFieldAccess {
        left: Box<Option<ExprBox>>,
        right: Box<ExprBox>,
    },
    ForLoop {
        init: Box<Ast>,
        test: Box<ExprBox>,
        on_loop: Box<ExprBox>,
        body: Block,
    },
    WhileLoop {
        test: Box<ExprBox>,
        body: Block,
    },
    InfLoop {
        body: Block,
    },
    If(IfExpr),
    Struct {
        path: Path,
        fields: HashMap<Ident, ExprBox>,
    },
    Return(Box<Option<ExprBox>>),
    Break,
    Continue,
    Cast {
        left: Box<ExprBox>,
        right: Box<TypeBox>
    },
}

impl ExprBox {
    pub fn unwrap_path(&self) -> Path {
        let Expr::Path(p) = self.inner.clone() else {panic!("Unwrapping")};
        p.clone()
    }
}



#[derive(Debug, Clone)]
pub struct CallParams(pub Vec<ExprBox>);

#[derive(Debug, Clone)]
pub struct Block(pub Vec<Ast>);

#[derive(Debug, Clone)]
pub struct Path(pub Vec<Ident>);


#[derive(Debug, Clone)]
pub struct IfExpr {
    pub test: Box<ExprBox>,
    pub body: Block,
    pub else_if: Option<IfBranchExpr>
}

#[derive(Debug, Clone)]
pub enum IfBranchExpr {
    ElseIf(Box<IfExpr>),
    Else(Block)
}

